@@ -24,6 +24,9 @@ class FullModelSchema(JsonSchemaOut):
         return len(df["time"].unique()) == 1
 
 
+FullModelDataFrame = DataFrame[FullModelSchema]
+
+
 class MridResourceSchema(JsonSchemaOut):
     """
     Common class for resources with an mrid as index
@@ -52,6 +55,9 @@ class MarketDatesSchema(NamedResourceSchema):
     activation_date: Series[dt.datetime] = pa.Field()
 
 
+MarketDatesDataFrame = DataFrame[MarketDatesSchema]
+
+
 class BusDataSchema(JsonSchemaOut):
     node: Index[str] = pa.Field(unique=True)
     name: Series[str] = pa.Field()
@@ -60,6 +66,8 @@ class BusDataSchema(JsonSchemaOut):
     station: Series[str] = pa.Field()
     bidzone: Series[str] = pa.Field(nullable=True)
     sv_voltage: Series[float] = pa.Field(nullable=True)
+    island: Series[str] = pa.Field()
+    is_swing_bus: Series[bool] = pa.Field()
 
 
 BusDataFrame = DataFrame[BusDataSchema]
@@ -75,6 +83,9 @@ class LoadsSchema(NamedResourceSchema):
     station_group: Series[str] = pa.Field(nullable=True)
 
 
+LoadsDataFrame = DataFrame[LoadsSchema]
+
+
 class WindGeneratingUnitsSchema(NamedMarketResourceSchema):
     station_group: Series[str] = pa.Field(nullable=True)
     min_p: Series[float] = pa.Field()
@@ -82,6 +93,9 @@ class WindGeneratingUnitsSchema(NamedMarketResourceSchema):
     plant_mrid: Series[str] = pa.Field(nullable=True)
 
 
+WindGeneratingUnitsDataFrame = DataFrame[WindGeneratingUnitsSchema]
+
+
 class SynchronousMachinesSchema(NamedMarketResourceSchema):
     allocationmax: Series[float] = pa.Field(nullable=True)
     node: Series[str] = pa.Field()
@@ -98,11 +112,17 @@ class SynchronousMachinesSchema(NamedMarketResourceSchema):
     q: Series[float] = pa.Field(nullable=True)
 
 
+SynchronousMachinesDataFrame = DataFrame[SynchronousMachinesSchema]
+
+
 class ConnectionsSchema(MridResourceSchema):
     t_mrid_1: Series[str] = pa.Field()
     t_mrid_2: Series[str] = pa.Field()
 
 
+ConnectionsDataFrame = DataFrame[ConnectionsSchema]
+
+
 class BordersSchema(NamedMarketResourceSchema):
     area_1: Series[str] = pa.Field()
     area_2: Series[str] = pa.Field()
@@ -110,12 +130,18 @@ class BordersSchema(NamedMarketResourceSchema):
     t_mrid_2: Series[str] = pa.Field()
 
 
+BordersDataFrame = DataFrame[BordersSchema]
+
+
 class ExchangeSchema(NamedMarketResourceSchema):
     node: Series[str] = pa.Field()
     status: Series[bool] = pa.Field()
     p: Series[float] = pa.Field()
 
 
+ExchangeDataFrame = DataFrame[ExchangeSchema]
+
+
 class ConvertersSchema(NamedResourceSchema):
     alias: Series[str] = pa.Field(nullable=True)
     station: Series[str] = pa.Field()
@@ -123,11 +149,17 @@ class ConvertersSchema(NamedResourceSchema):
     node: Series[str] = pa.Field()
 
 
+ConvertersDataFrame = DataFrame[ConvertersSchema]
+
+
 class TransfConToConverterSchema(NamedResourceSchema):
     t_mrid: Series[str] = pa.Field()
     p_mrid: Series[str] = pa.Field()
 
 
+TransfConToConverterDataFrame = DataFrame[TransfConToConverterSchema]
+
+
 class CoordinatesSchema(JsonSchemaOut):
     mrid: Series[str] = pa.Field()
     x: Series[str] = pa.Field()
@@ -136,6 +168,9 @@ class CoordinatesSchema(JsonSchemaOut):
     rdf_type: Series[pd.CategoricalDtype] = pa.Field()
 
 
+CoordinatesDataFrame = DataFrame[CoordinatesSchema]
+
+
 class BranchComponentSchema(NamedResourceSchema):
     bidzone_1: Series[str] = pa.Field(nullable=True)
     bidzone_2: Series[str] = pa.Field(nullable=True)
@@ -150,16 +185,25 @@ class BranchComponentSchema(NamedResourceSchema):
     x: Series[float] = pa.Field()
 
 
+BranchComponentDataFrame = DataFrame[BranchComponentSchema]
+
+
 class ShuntComponentSchema(BranchComponentSchema):
     b: Series[float] = pa.Field()
     g: Series[float] = pa.Field()
 
 
+ShuntComponentDataFrame = DataFrame[ShuntComponentSchema]
+
+
 class AcLinesSchema(ShuntComponentSchema):
     length: Series[float] = pa.Field()
     g: Series[float] = pa.Field(nullable=True)
 
 
+AcLinesDataFrame = DataFrame[AcLinesSchema]
+
+
 class TransformersSchema(JsonSchemaOut):
     name: Series[str] = pa.Field()
     p_mrid: Series[str] = pa.Field()
@@ -172,37 +216,58 @@ class TransformersSchema(JsonSchemaOut):
     rate: Series[float] = pa.Field(nullable=True)
 
 
+TransformersDataFrame = DataFrame[TransformersSchema]
+
+
 class TransformerWindingSchema(ShuntComponentSchema):
     angle: Series[float] = pa.Field()
     ratio: Series[float] = pa.Field()
 
 
+TransformerWindingDataFrame = DataFrame[TransformerWindingSchema]
+
+
 class SubstationVoltageSchema(JsonSchemaOut):
     substation: Index[str] = pa.Field()
     container: Series[str] = pa.Field()
     v: Series[float] = pa.Field()
 
 
+SubstationVoltageDataFrame = DataFrame[SubstationVoltageSchema]
+
+
 class DisconnectedSchema(JsonSchemaOut):
     mrid: Series[str] = pa.Field(unique=True)
 
 
+DisconnectedDataFrame = DataFrame[DisconnectedSchema]
+
+
 class PowerFlowSchema(MridResourceSchema):
     p: Series[float] = pa.Field()
     q: Series[float] = pa.Field()
     in_service: Series[bool] = pa.Field()
 
 
+PowerFlowDataFrame = DataFrame[PowerFlowSchema]
+
+
 class BranchWithdrawSchema(MridResourceSchema):
     node: Series[str] = pa.Field()
     p: Series[float] = pa.Field()
     q: Series[float] = pa.Field()
 
 
+BranchWithdrawDataFrame = DataFrame[BranchWithdrawSchema]
+
+
 class DcActiveFlowSchema(MridResourceSchema):
     p: Series[float] = pa.Field()
 
 
+DcActiveFlowDataFrame = DataFrame[DcActiveFlowSchema]
+
+
 class RegionsSchema(MridResourceSchema):
     region: Series[str] = pa.Field()
     short_name: Series[str] = pa.Field(nullable=True)
@@ -211,12 +276,18 @@ class RegionsSchema(MridResourceSchema):
     region_name: Series[str] = pa.Field(nullable=True)
 
 
+RegionsDataFrame = DataFrame[RegionsSchema]
+
+
 class StationGroupCodeNameSchema(JsonSchemaOut):
     station_group: Index[str] = pa.Field(unique=True)
     name: Series[str] = pa.Field()
     alias_name: Series[str] = pa.Field(nullable=True)
 
 
+StationGroupCodeNameDataFrame = DataFrame[StationGroupCodeNameSchema]
+
+
 class HVDCBidzonesSchema(MridResourceSchema):
     from_area: Series[str] = pa.Field()
     to_area: Series[str] = pa.Field()
